#!/usr/bin/perl -T
# Copyright 2003 The Board of Trustees of 
# The Leland Stanford Junior University. All Rights Reserved.

# Communication to Jessica Shapiro, jessicas@stanford.edu

use CGI;
use lib "/var/www/cgi-bin/FoldMiner2"; 
use Scop qw(%classifications $VERSION);
use Expectation qw(%lambda %mu);

$ENV{'PATH'} = "/bin:/usr/bin:/usr/local/bin";
$EOL = "\r\n";

$lockprogram = "/usr/local/bin/lock2";
$touch = "/bin/touch";
$scopversion = $VERSION;
$MAX_TARGETS = 10;

#note: $$ is the process id for this run
$outputdir = "/var/www/html/FoldMiner2/results/$$"; #results for this run
$wwwoutput = "/FoldMiner2/results/$$"; #output directory for this run
$ID = $$;

#system commands
$mv = "/bin/mv";
$cp = "/bin/cp";
$echo="/bin/echo";

#check to see if outputdir already exists
my($done) = 0;
while(!$done) {
  if(! -e($outputdir)) {
    $done = 1;
  }else {
    $ID += 1;
    $outputdir = "/var/www/html/FoldMiner2/results/$ID";
    $wwwoutput = "/FoldMiner2/results/$ID";
  }
}

#read in some of the parameters
$q = new CGI;
$chain1 = $q->param('chain1');
$chain2 = $q->param('chain2');
$pdbname1 = $q->param('pdbname1');
$pdbname2 = $q->param('pdbname2');
$pdbpaste1 = $q->param('pdbpaste1');
$pdbpaste2 = $q->param('pdbpaste2');
$pdbfile1 = $q->param('pdbfile1'); #query pdbfile, if it was uploaded
$pdbfile2 = $q->param('pdbfile2'); #target pdbfile, if it was uploaded
$multsearch = $q->param('multsearch');
$searchpdb = $q->param('searchpdb');
$nonsequential = $q->param('sequential'); #if value==1,residues must be numbered in order only over SSEs; otherwise must be numbered in order over whole alignment
$distthreshold = $q->param('distance');
$seqpairs = $q->param('seqpairs');
$expectation = $q->param('expectation');
$exponent = $q->param('exponent');
$xval = $q->param('xval');
$chime = $q->param('chime');
$address = $q->param('mailaddress');

#check some inputs for security
my($temppdbname1,$temppdbname2);
$pdbname1 =~ tr/A-Z/a-z/;
$pdbname2 =~ tr/A-Z/a-z/;

if($pdbname1 =~ /^(.*)_([A-Za-z0-9])$/) {
  $pdbname1 = $1."-".$2;
}
if($pdbname2 =~ /^(.*)_([A-Za-z0-9])$/) {
  $pdbname2 = $1."-".$2;
}
if($pdbname1 ne "") {
  $temppdbname1 = $pdbname1;
  if(length($pdbname1) == 7) {
    $pdbname1 = securityCheckProteinName($pdbname1,1);
  }else {
    $pdbname1 = securityCheckProteinName($pdbname1,0);
  }
}
$temppdbname2=$pdbname2;
if($pdbname2 ne "" && !$searchpdb) {  
  if(length($pdbname2) == 7) {
    $pdbname2 = securityCheckProteinName($pdbname2,1);
  }else {
    $pdbname2 = securityCheckProteinName($pdbname2,0);
  }
}

if(defined($multsearch)) {
  if($multsearch =~ /^([10])$/) {
    $multsearch = $1;
  }else {
    redirectToErrorHandler($multsearch,"You have submitted a request to align a query structure to multiple target structures, but an internal variable indicating that you wish to perform this type of search has an unexpected value. If you are not submitting this search via the LOCK 2 homepage, please make sure you do not change the states of hidden variables.","","",$ID);
    exit(-1);
  }
}

if(defined($searchpdb)) {
  if($searchpdb =~ /^([10])$/) {
    
    $searchpdb = $1;
  }else {
    redirectToErrorHandler($searchpdb,"You have submitted a request to align a query structure to a database of target structures, but an internal variable indicating that you wish to perform this type of search has an unexpected value. If you are not submitting this search via the LOCK 2 homepage, please make sure you do not change the states of hidden variables.","","",$ID);
    exit(-1);
  }
}



if(defined($pdbname1) && $pdbname1 ne "") {
  if($pdbname1 eq "") {
    redirectToErrorHandler($temppdbname1,"Query PDB or SCOP id is missing or incorrectly formatted","d1f5wa1,1f5w,1f5w-A","query",$ID);
    exit(-1);
  }
}else {

  my($oldforwardslash) = "\/";
  my($newforwardslash) = "\\/";
  my($oldbackslash) = "\\";
  my($newbackslash) = "\\\\";
  $pdbfile1 =~ sm|$oldforwardslash|$newforwardslash|g;
  $pdbfile1 =~ sm|$oldbackslash|$newbackslash|g;
#  if($pdbfile1 =~ /^([-:\\\/\w_\.\s]+)$/) { #filename may only have word characters (alphanumeric + "_"), "-", and "."
  if($pdbfile1 =~ /^([^\*]*)$/) {
    $pdbfile1 = $1;
  }else {
    redirectToErrorHandler($pdbfile1,"The name of the query structure file to be uploaded contains illegal characters.","","query",$ID);
    exit(-1);
  }
}
if(defined($pdbname2) and $pdbname2 ne "") {
  if($pdbname2 eq "" && !($searchpdb || $multsearch) && !(defined($pdbfile2)||$pdbfile2=="")) {
    redirectToErrorHandler($temppdbname2,"Target PDB or SCOP id is missing or incorrectly formatted","d1f5wa1,1f5w,1f5w-A","target",$ID);
    exit(-1);
  }
}elsif(!($searchpdb || $multsearch)) {
  my($oldforwardslash) = "\/";
  my($newforwardslash) = "\\/";
  my($oldbackslash) = "\\";
  my($newbackslash) = "\\\\";
  $pdbfile2 =~ sm|$oldforwardslash|$newforwardslash|g;
  $pdbfile2 =~ sm|$oldbackslash|$newbackslash|g;
#  if($pdbfile2 =~ /^([-:\\\/\w_\.\s]+)$/) { #filename may only have word characters (alphanumeric + "_"), "-", and "."
  if($pdbfile2 =~ /^([^\*]*)$/) {
    $pdbfile2 = $1;
  }else {
    redirectToErrorHandler($pdbfile2,"The name of the target structure file to be uploaded is contains illegal characters.","","target",$ID);
    exit(-1);
  }
}

#check chains
my($tempqchain,$temptchain);
if(length($chain1) > 0) {
  if($chain1 =~ /^([A-Za-z0-9])$/) {
    $chain1 = $1;
    $tempqchain = $chain1;
  }else {
    if($chain1 eq " ") {
      $chain1 = "";
      $tempqchain = "";
    }else {
      redirectToErrorHandler($chain1,"The specified chain for the query structure contains illegal characters. Only letters and numbers are permited. It is not necessary to specify a chain if the query structure contains only one chain or if you wish to superimpose all chains.","A,a","idcodes",$ID);
      exit(-1); 
    }
  }
}

if(length($chain2) > 0) {
  if($chain2 =~ /^([A-Za-z0-9])$/) {
    $chain2 = $1; $temptchain = $chain2;
  }else {
    if($chain2 eq " ") {
      $chain2 = ""; $temptchain = "";
    }
    redirectToErrorHandler($chain2,"The specified chain for the target structure contains illegal characters. Only letters and numbers are permited. It is not necessary to specify a chain if the target structure contains only one chain or if you wish to superimpose all chains.","A,a","idcodes",$ID);
    exit(-1); 
  }
}



if(defined($nonsequential)) {
  if($nonsequential =~ /^([01])$/) {
    $nonsequential = $1;
  }else {
    redirectToErrorHandler("","An internal variable controlling a LOCK 2 alignment option (sequential residue numbering) has an unexpected value. If you are not submitting this search via the LOCK 2 homepage, please make sure you submit the value associated with the appropriate 'Sequential Residue Numbering' radio button.","","resnumbering",$ID);
    exit(-1);
  }
}

if(defined($distthreshold)) {
  if($distthreshold =~ /^([0-9]*)(\.[0-9]*)?$/) {
    $distthreshold = $1.$2;
  }else {
    redirectToErrorHandler($distthreshold,"The distance threshold you submitted has an unexpected value. The value must be a positive number.","3,3.5,.5","distanceThreshold",$ID);
    exit(-1);
  }
}

if(defined($seqpairs)) {
  if($seqpairs =~ /^([01])$/) {
    $seqpairs = $1;
  }else {
    redirectToErrorHandler("","An internal variable controlling a LOCK 2 alignment option (geometric hashing options) has an unexpected value. If you are not submitting this search via the LOCK 2 homepage, please make sure you submit the value associated with the appropriate 'Geometric Hashing' radio button.","","seqpairs",$ID);
    exit(-1);
  }
}

if(defined($expectation)) {
  if($expectation =~  /^([0-9]*)(\.[0-9]*)?$/) {
    $expectation = $1.$2;
  }elsif($expectation =~ /^([0-9]+e-?[0-9]+)?$/){
    $expectation = $1;
  }else {
    redirectToErrorHandler($expectation,"The mantissa value you submitted has an unexpected value. The value must be a positive number.","10,0.1,.1","expectation",$ID);
    exit(-1);
  }
}
$base = $expectation;
if(defined($exponent)) {
  if($exponent =~  /^(-?[0-9]*)(\.[0-9]*)?$/) {
    $exponent = $1.$2;
  }else {
    redirectToErrorHandler($exponent,"The exponent portion of the expectation value you submitted has an unexpected value. The value must be a positive number.","10,0.1,.1","expectation",$ID);
    exit(-1);
  }
}

if(defined($xval)) {
  if(($xval =~ /^([0-9]?\.[0-9]+)$/) || $xval < 0 || $xval > 1) {
    $xval = $1;

  }else {
    redirectToErrorHandler($xval,"The value you submitted for the conservation profile's contribution to recalculated SSE scores has an unexpected value. The value must be a number between 0 and 1.","0,0.5,.5,1.0","conservation",$ID);
    exit(-1);
  }
}

  
  if(defined($address) && $address ne "") {
    if($address =~ /^(NONE)$/) {
      $address = $1;
    }else {
      if($address =~ m|^[\s\n\r]*\"?([^\"]+\"?@[^ <>\t].[^ <>\t][^ <>\t]+)[\s\n\r]*$|) {
	$address = $1;
      }else {
	redirectToErrorHandler($address,"The email address you entered does not appear to be valid",'jessicas\@stanford.edu',,$ID);
      }
    }
  }else {
    $address = "NONE";
  }

if(defined($chime)) {
  if($chime =~ /^([01])$/) {
    $chime = $1;
  }else {
    redirectToErrorHandler("","An internal variable has an unexpected value. If you are not submitting this search via the LOCK 2 homepage, please make sure that you do not change the values of hidden variables.","","",$ID);    
    exit(-1);
  }
}

my($error);
$error = system("/bin/mkdir", "$outputdir");

#no longer using minret: set to zero until we get rid of it
my($minret) = 0;

my($seqpairsoption);
if($seqpairs == 1) {
  $seqpairsoption = "-s";
}else {
  $seqpairsoption = "";
}


my($query,$target,$querychain,$querychainopt,$targetchain,$targetchainopt,$querypdblink,$queryoutname,$shortpdbname,$shortqueryscopname,$shorttargetscopname,$targetpdblink,$targetoutname);
my($qfilehandle,$tfilehandle,$querymail,$targetmail);
my($line,@fields,@temp);
if ($chain1 ne "") {
    @temp = split //, $chain1;
    if ($temp[0] ne " ") {
	$querychain = $temp[0];
	$querychain =~ tr/a-z/A-Z/;
	$querychainopt = "-qc";
    }
}else {
  $querychainopt = ""; $querychain = "";
}
			

if ($pdbname1 ne "") {
    $query = $pdbname1;
    if(length($query) == 7) {
      ($shortpdbname) = $query =~/^.(....)/;
      ($shortqueryscopname) = $query =~/^(.......)/;
      ($tempqchain) = $query =~ /^.....(.)/;
      if($tempqchain eq "_") {
	$tempqchain = "";
      }
      $querypdblink = "http://scop.berkeley.edu/search.cgi?sid=$shortqueryscopname"; 
      $queryoutname = $shortqueryscopname;
    }else {
      ($shortpdbname) = $query =~ /^(....)/;

      if($query =~ /^....-(.)/){
	$tempqchain = $1;
	$querychainopt = "-qc";
	$shortqueryscopname = "d".$shortpdbname.$tempqchain."_";
      }elsif($query =~ /^....(.)/) {
	$tempqchain = $1;
	$querychainopt = "-qc";
	$shortqueryscopname = "d".$shortpdbname.$tempqchain."_";
      }else {
	if($chain1 ne "" && $chain1 ne " ") {
	  $shortqueryscopname = "d".$shortpdbname.$chain1."_";
	}else {
	  $shortqueryscopname = "d".$shortpdbname."__";	  
	}
      }
      $shortqueryscopname =~ tr/A-Z/a-z/;

    if (defined $tempqchain && $tempqchain ne "") { #if chain was defined in pdbname1 instead of chain1
      $tempqchain =~ tr/a-z/A-Z/;
      $queryoutname = "$shortpdbname\-$tempqchain";
      $query = $shortpdbname;
      $querychain = $tempqchain;
      } else {
	if($querychain ne "") {
	  $queryoutname = "$shortpdbname\-$querychain";
	  $query = $queryoutname;
	}else {
	  $queryoutname = $shortpdbname;
	  $query = $queryoutname;
	}
      }
    
      $querypdblink = "/cgi-bin/FoldMiner2/browse2.cgi?pdbname=$queryoutname&resultsdir=$outputdir&wwwdir=$wwwoutput"; 
    }
  }elsif (defined($pdbfile1)) {
    $query = "$outputdir/queryfile.pdb";
    $qfilehandle = $q->upload('pdbfile1');
    open(qfile, ">$query");

    while($line = <$qfilehandle>) {
	$line =~ tr/\r[^\n]/\n/;
      print qfile $line;
    }

    close(qfile);
    $queryoutname = "query";

    $querypdblink = "/cgi-bin/FoldMiner2/browse2.cgi?pdbname=queryfile&resultsdir=$outputdir&wwwdir=$wwwoutput";
  }
  else {
    redirectToErrorHandler("","You must either specify the query structure by its SCOP or PDB id or submit a PDB file to be uploaded","","query",$ID);
    exit(-1);
}

my($queryclassification,$queryclass,$queryfold,$querysuperfam,$queryfam);
  if($searchpdb == 0) { #will need SCOP classification of query, if available
    if(length($query) == 7) {
      if(defined($classifications{$shortqueryscopname})) {
	$queryclassification = $classifications{$shortqueryscopname};
	($queryclass,$queryfold,$querysuperfam,$queryfam) = split(/\./,$queryclassification);
	$queryfold = $queryclass.".".$queryfold;
      }else {

      }
    }else {
      if($queryoutname ne "query") {
	my($domain,$scopid,$pdbid,$chain);
	$queryclassification = "";
	for $domain (keys(%classifications)) {
	  if($domain eq $shortqueryscopname) {
	    $queryclassification = $classifications{$domain};
	    ($queryclass,$queryfold,$querysuperfam,$queryfam) = split(/\./,$queryclassification);
	    $queryfold = $queryclass.".".$queryfold;
	    last;
	  }
	}
	if($queryclassification eq "") {
	  $queryfold = "all";
	}
      }else {
	$queryfold = "all";
      }
    }
  }



if (($multsearch == 0) && ($searchpdb == 0)) {
    if ($chain2 ne "") {
	@temp = split //, $chain2;
	if ($temp[0] ne " ") {
	    $targetchain = $temp[0];
	    $targetchain =~ tr/a-z/A-Z/;
	    $targetchainopt = "-tc";
	}
    }else {
      $targetchainopt = ""; $targetchain = "";
    }


    if ($pdbname2 ne "") {
	$target = $pdbname2;
	if(length($target) == 7) {
	  ($shortpdbname) = $target =~/^.(....)/;
	  ($shorttargetscopname) = $target =~/^(.......)/;
	  ($temptchain) = $target =~ /^.....(.)/;
	  if($temptchain eq "_") {
	    $temptchain="";
	  }
	  $targetpdblink = "http://scop.berkeley.edu/search.cgi?sid=$shorttargetscopname"; 
	  $targetoutname = $shorttargetscopname;
	}else {
	  ($shortpdbname) = $target =~ /^(....)/;
	  if($target =~ /^....-(.)/) {
	    $temptchain = $1;
	    $targetchainopt = "-tc";
	  }elsif($target =~/^....(.)/) {
	    $temptchain = $1;
	    $targetchainopt = "-tc";
	  }

	  if (defined $temptchain && $temptchain ne "") {
	    $temptchain =~ tr/a-z/A-Z/;	
	    $targetoutname = "$shortpdbname\-$temptchain";
	    $target = $shortpdbname;
	    $targetchain = $temptchain;
	  } else {
	    if($targetchain ne "") {
	      $targetoutname = "$shortpdbname-\$targetchain";
	      $target = $targetoutname;
	    }else {
	      $targetoutname = $shortpdbname;
	      $target = $targetoutname;
	    }
	  }
	  $targetpdblink = "/cgi-bin/FoldMiner2/browse2.cgi?pdbname=$targetoutname&resultsdir=$outputdir&wwwdir=$wwwoutput";
	}

      }elsif (defined($pdbfile2)) {
	$target = "$outputdir/targetfile.pdb";
	$tfilehandle = $q->upload('pdbfile2');
	open(tfile, ">$target");
	if($line =~ /^LOCK_LIST/) {
	  redirectToErrorHandler("","To align a query structure to multiple targets, please select the \"Multiple Pairs\" button from the <a href=/FoldMiner2/>LOCK 2.0 homepage</a>.","","target",$ID);
	  exit(-1);
	}
	print tfile $line;
	while($line = <$tfilehandle>) {
	$line =~ tr/\r[^\n]/\n/;
	  print tfile $line;
	}
	close(tfile);
	$targetoutname = "target";
	
	$targetpdblink = "/cgi-bin/FoldMiner2/browse2.cgi?pdbname=targetfile&resultsdir=$outputdir&wwwdir=$wwwoutput";
    }
    else {
      redirectToErrorHandler("","You must either specify the target structure by its SCOP or PDB id or submit a PDB file to be uploaded","","target",$ID);
      exit(-1);
    }

    $querymail = $query;
    $targetmail = $target;

    runlock_single();

}
elsif ($multsearch == 1) {
    if ($pdbpaste2 ne "") {
      @targets = split(/\n/,$pdbpaste2); #count number of targets
      $numtargets = scalar(@targets);
      if($numtargets > $MAX_TARGETS) {
	redirectToErrorHandler($pdbpaste2,"Only $MAX_TARGETS may be entered when aligning a query structure to multiple target structures. You entered $numtargets. If you would like to align your query structure to a large number of target structures, consider choosing the 'Search SCOP Subsets' option.","","searchsubset",$ID);
	exit(-1);
      }
      for($i=0;$i<=$#targets;$i++) {
	$targets[$i] =~ tr/A-Z/a-z/;
	$targets[$i] =~ s/ //g;
	$targets[$i] =~ s/[\r\n]//g;
	$temp = $targets[$i];
	$targets[$i] = $temp;
	$len = length($temp);
	if(length($targets[$i]) == 7) {

	  $targets[$i] = securityCheckProteinName($targets[$i],1);
	  if($targets[$i] eq "") {
	    redirectToErrorHandler($temp,"Target appears to be a SCOP id, but is incorrectly formatted","d1f5wa1,d1neu__","target",$ID);
	    exit(-1);
	  }
	}else {

	  if($targets[$i] =~ /^(.*)_([A-Za-z0-9])$/) {
	    $targets[$i] = $1."-".$2;
	  }
	  if($targets[$i] =~ /^(....)(.)$/) {
	    $targets[$i] = $1."-".$2;
	  }
	  $targets[$i] = securityCheckProteinName($targets[$i],0);
	  if($targets[$i] eq "") {
	    redirectToErrorHandler($temp,"Target PDB id is incorrectly formatted, or target does not appear to be a SCOP or PDB id.","1f5w,1f5w-A","target",$ID);
	    exit(-1);
	  }
	}

      }
      $target = "$outputdir/targetfile";
      open(tfile, ">$target");
      print tfile "LOCK_LIST$EOL$EOL";
      print tfile $pdbpaste2;
      close(tfile);
    }
    else {
      redirectToErrorHandler("","You have elected to align a query structure to multiple target structures, but you have not submitted a list of targets.","d1f5wa1\n1f5w\n1f5w\n1f5w-A","target",$ID);
	exit(-1);
    }    

    $querymail = $query;
    $targetmail = $target;

    runlock_mult();

}

elsif($searchpdb == 1) { #searching against a scop database
  #get the target databasefile
  $filterType = $q->param('filterType');
  if($filterType =~ /^([01])$/) {
    $filterType = $1;
  }else {
    redirectToErrorHandler("","An internal variable controlling the type of SCOP target database selected has an unexpected value. If you are not submitting this search via the LOCK 2 homepage, please make sure that your value for the filterType variable corresponds to the appropriate radio button.","","searchsubsets",$ID);    
    exit(-1);
  }

  if($filterType==0) { #using percent identity
    $percentID = $q->param('percentID');
    if($percentID =~ /^([0-9]{2,3})$/) {
      $percentID = $1;
    }else {
      redirectToErrorHandler($percentID,"An internal variable controlling the SCOP target database selected has an unexpected value. If you are not submitting this search via the LOCK 2 homepage, please make sure that your value for the percentID variable corresponds to one of the options available on the LOCK 2 web pages.","","searchsubsets",$ID);    
      exit(-1);
    }
    $target = "/var/www/html/FoldMiner2/targetdb/astral-scopdom-seqres-gd-sel-gs-bib-".$percentID."-".$scopversion.".id";
  }elsif($filterType==1) {
    $scopLevel = $q->param('scopLevel');

    if(!($scopLevel eq "cl" || $scopLevel eq "cf" || $scopLevel eq "sf" || $scopLevel eq "fa" || $scopLevel eq "dm")) {
      redirectToErrorHandler($percentID,"An internal variable controlling the SCOP target database selected has an unexpected value. If you are not submitting this search via the LOCK 2 homepage, please make sure that your value for the scopLevel variable corresponds to one of the options available on the LOCK 2 web pages.","","searchsubsets",$ID);    
      exit(-1);
    }
    $target = "/var/www/html/FoldMiner2/targetdb/astral-scopdom-seqres-gd-sel-gs-sc-".$scopLevel."-".$scopversion.".id";
  }

  $querymail = $query;
  $targetmail = $target;

  runlock_search();
}

print "</body></html>$EOL";


exit(0);



#############################################

sub runlock_single() {
  system("$cp","/var/www/html/FoldMiner2/waiting.html","$outputdir/results.html");
  open(OLD_STDOUT,">&STDOUT"); #save STDOUT filehandle
  open(STDOUT, ">$outputdir/logfile");
  select(STDOUT); $|=1;
  system($lockprogram, "-b", "-d", $distthreshold, "-q", $query, $querychainopt,$querychain, "-t", $target,$targetchainopt,$targetchain, "-o", $outputdir, $seqpairsoption);
  system($touch,"$outputdir/done");
  close(STDOUT);
  
  open(LOG, "$outputdir/logfile");
  @lines = <LOG>;
  close(LOG);

  #check to make sure query file was found
  if($lines[0] =~ /^attempting to find file: $query/) {
    if($lines[2] =~ /Could not find file $query/) {
      open(STDOUT, ">&OLD_STDOUT");
      redirectToErrorHandler($pdbname1,"The query PDB file could not be located in our local database. If there are no errors in your input, please repeat your request and upload a PDB file.","d1f5wa1,1f5w,1f5w-A","query",$ID);
      exit(-1);
    }
  }

  #now check to see if target was not found
  for($i=3;$i<=$#lines;$i++) {
    if($lines[$i] =~ /^attempting to find file: $target/) {
      if($lines[$i+2] =~/Could not find file $target/) {
	open(STDOUT, ">&OLD_STDOUT");
	redirectToErrorHandler($pdbname2,"The target PDB file could not be located in our local database. If there are no errors in your input, please repeat your request and upload a PDB file.","d1f5wa1,1f5w,1f5w-A","target",$ID);
	exit(-1);
      }
    }
  }
  
  
  #this sorts search.out by score; include a tiebreaker column?
  if (-e "$outputdir/search\.out") {
    open(STDOUT, ">$outputdir/search\.sort");
    select(STDOUT); $|=1;
    system("/bin/sort -t\\| -k2,2n -k3,3nr $outputdir/search\.out");

  }else {
    close(STDOUT);
    open(STDOUT, ">&OLD_STDOUT");
    $errorline = "";
    for($i=0;$i<=$#lines;$i++) {
      if($lines[$i] =~ /attempting to find file:\s*(.*)\n/) {
	$filetofind = $1;
      }elsif($lines[$i] =~ /Query = (.*) \*\*/) {
	$this_query = $1;
	$filetofind = $1;
      }elsif($lines[$i] =~ /Target = (.*) \*\*/) {
	$this_target = $1;
	$filetofind = $1;
      }else {
	$filetofind = "either the query or target structure";
      }
      if($lines[$i] =~ /(Sorry\.?\s*)(.*)/) {
	if($lines[$i] =~ /Could not find a DSSP file/) {
	  $errorline .= " Could not find or create a DSSP file for $filetofind. It is necessary to produce a DSSP file in order to obtain secondary structure definitions. If SHEET and/or HELIX headers are included in the PDB file, it will not be necessary to create a DSSP file. Please try creating these headers and uploading the PDB file.";
      }elsif($lines[$i] =~ /No CA atoms with chain identifier/) {
	  $errorline .= $lines[$i];
      }else {
	  $errorline .= " Could not find PDB file for $filetofind or could not create a DSSP file. If you specified a structure using a PDB or SCOP identifier, please try uploading the PDB file. Otherwise, please bear in mind that it is necessary to produce a DSSP file in order to obtain secondary structure definitions. If SHEET and/or HELIX headers are included in the PDB file, it will not be necessary to create a DSSP file.";
	}
      }
    }
    redirectToErrorHandler("","$errorline","","",$ID);
    exit(-1);
  }
  close(STDOUT);
  open(STDOUT, ">&OLD_STDOUT"); #revert to original STDOUT
  open(htmlfile, ">$outputdir/tmp");
  printHtmlHeader();
    print htmlfile "View the aligned structures in <a href=http://www.umass.edu/microbio/rasmol/><b>RasMol</b></a> , <a href=http://www.mdl.com/><b>Chime</b></a> , or any other molecular viewer of your choice.  For best results in RasMol or Chime, select <i>display:backbone</i> and <i>color:chain</i>.  This will color the <font color=#0000ff>query</font> molecule in <font color=#0000ff>blue</font> and the <font color=#ff0000>target</font> in <font color=#ff0000>red</font>.<br><br><hr><br>$EOL"; 

    print htmlfile "<p><b>LOCK 2 search results for query: <a href=$querypdblink target=_blank>$queryoutname</a></b><br><br>$EOL";

    $errorString = `/bin/grep -i sorry $outputdir/logfile`;

    if ($errorString ne "") {
      open(errorfile, ">$outputdir/errorfile");
      print errorfile $errorString;
      close(errorfile);
      print htmlfile "<b>!!! Some <a href=$wwwoutput/errorfile>ERROR(s)</a> occurred !!!</b><br><br>$EOL";
    }
	

  print htmlfile "<a href=$wwwoutput/logfile>Execution Log</a><br>$EOL";
  print htmlfile "<a href=$wwwoutput/search\.sort>Search results (non-html)</a><br>$EOL";

  print htmlfile"<br>$EOL";
	

  open(INFILE, "<$outputdir/search\.sort");
  print htmlfile "<table width=100% cellpadding=0 cellspacing=0 border=1>$EOL";
  print htmlfile "<tr>$EOL";
  print htmlfile "<td align=left> <b>Target</b> </td>$EOL";
  print htmlfile "<td align=center> <b><a href=\"javascript:launchHelpWindow('resultsformats')\" onMouseOver=\"status='Click for help';return true\" onMouseOut=\"status='';return true\" class=\"helplink\">Results</a></b> </td>$EOL";
  print htmlfile "<td align = center> <b><a href=\"javascript:launchHelpWindow('alignmentscore')\" onMouseOver=\"status='Click for help';return true\" onMouseOut=\"status='';return true\" class=\"helplink\">Score</a></b> </td>$EOL"; #added
  print htmlfile "<td align = center> <b><a href=\"javascript:launchHelpWindow('significance')\" onMouseOver=\"status='Click for help';return true\" onMouseOut=\"status='';return true\" class=\"helplink\">P value</a></b></td>$EOL"; #added
  print htmlfile "<td align = center> <b>SSEs Aligned</b></td>$EOL"; #added
  print htmlfile "<td align=center> <b>CA Atoms<br>Aligned</b> </td>$EOL";
  print htmlfile "<td align=center> <b>RMSD</b> </td>$EOL";
  print htmlfile "<td align=left> <b>PDB HEADER</b> </td>$EOL";
  print htmlfile "</tr>$EOL";
  
  
  
  $line = <INFILE>;
  
  chop($line);
  @fields = split /\|/, $line;
  $pval = &expectation_function($lambda{$queryfold},$mu{$queryfold},$fields[1]);
  print htmlfile "<tr>$EOL";
  print htmlfile "<td align=left>$EOL";
  print htmlfile "<a href=$targetpdblink>$targetoutname</a></td>$EOL";
  print htmlfile "<td align=center>$EOL";
  print htmlfile "<a href=$wwwoutput/$queryoutname\_$targetoutname\.out>Text</a>";
  print htmlfile " , ";
  print htmlfile "<a href=/cgi-bin/FoldMiner2/newpdb.cgi?yourfile=$queryoutname\_$targetoutname\.out\&outfile=$queryoutname\_$targetoutname\.pdb\&outdir=$outputdir\&wwwdir=$wwwoutput\&chime=0>PDB</a>";
  print htmlfile " , ";
  print htmlfile "<a href=/cgi-bin/FoldMiner2/newpdb.cgi?yourfile=$queryoutname\_$targetoutname\.out\&outfile=$queryoutname\_$targetoutname\.pdb\&outdir=$outputdir\&wwwdir=$wwwoutput\&chime=1>Chime</a>";
  
  print htmlfile "</td>$EOL";
  print htmlfile "<td align=center>$EOL";
  print htmlfile "$fields[1]</td>$EOL";
  printf(htmlfile "<td align=center>%8.3g</td>$EOL",$pval);
  print htmlfile "<td align=center>$EOL";
  print htmlfile "$fields[2]</td>$EOL";
  print htmlfile "<td align=center>$EOL";
  print htmlfile "$fields[3]</td>$EOL";
  print htmlfile "<td align=center>$EOL";
  print htmlfile "$fields[4]</td>$EOL";
  print htmlfile "<td align=left>$EOL";
  print htmlfile "$fields[5]</td>$EOL";
  
  print htmlfile "</tr>$EOL";
  
  print htmlfile "</table>$EOL";
  close(htmlfile);
  close(INFILE);
  system("$mv","$outputdir/tmp","$outputdir/results\.html");
  printHeader();
}


sub runlock_mult() {
  
  system("$cp","/var/www/html/FoldMiner2/waiting.html","$outputdir/results.html");
  printMultSearchHeader();

  open(OLD_STDOUT,">&STDOUT"); #save STDOUT filehandle
  open(STDOUT, ">$outputdir/logfile");
  select(STDOUT); $|=1;
  system($lockprogram, "-b", "-d", $distthreshold, "-q", $query, $querychainopt, $querychain,"-t", $target, $targetchainopt, $targetchain,"-o", $outputdir, $seqpairsoption);
#  system("$lockprogram", "-b", "-q", $queryoutname, $querychainopt,$querychain "-t", $target, "-o", $outputdir, $coreoption);
  system($touch,"$outputdir/done");
  close(STDOUT);


  #check to make sure query was found
  open(LOG, "$outputdir/logfile");
  @lines = <LOG>;
  close(LOG);
  if($lines[0] =~ /^attempting to find file: $query/) {
    if($lines[2] =~ /Could not find file $query/) {
      open(STDOUT, ">&OLD_STDOUT");
      redirectToErrorHandler($pdbname1,"The query PDB file could not be located in our local database. If there are no errors in your input, please repeat your request and upload a PDB file.","d1f5wa1,1f5w,1f5w-A","query",$ID);
      exit(-1);
    }
  }


  #check to see if any targets were not found
  @errors = map(/Could not find file (.*)\./,@lines);
  if(scalar(@errors) >=1) {
    open(STDOUT, ">&OLD_STDOUT");
    $missingtargets = "";
    foreach $targeterror (@errors) {
      $missingtargets .="$targeterror, ";
    }
    $missingtargets = substr($missingtargets,0,length($missingtargets)-2);
    redirectToErrorHandler($missingtargets,"At least one of the target PDB files could not be located in our local database. If there are no errors in your input, please return to the <a href=\"\/LOCK\">LOCK 2 homepage<\/a> and run these alignments in single pair mode by uploading the target PDB files. You may also resubmit this request without the following targets: $missingtargets","d1f5wa1,1f5w,1f5w-A","target",$ID);
    exit(-1);
  }


  if (-e "$outputdir/search\.out") {
    open(STDOUT, ">$outputdir/search\.sort");
    select(STDOUT); $|=1;
    system("/bin/sort", "-n", "-r", "+1", "-2", "$outputdir/search\.out"); 
  }
  
  close(STDOUT);

  open(STDOUT, ">&OLD_STDOUT"); #revert to original STDOUT

  open(htmlfile, ">$outputdir/tmp");    
  printHtmlHeader();


  print htmlfile "View the aligned structures in <a href=http://www.umass.edu/microbio/rasmol/><b>RasMol</b></a> , <a href=http://www.mdl.com/><b>Chime</b></a> , or any other molecular viewer of your choice.  For best results in RasMol or Chime, select <i>display:backbone</i> and <i>color:chain</i>.  This will color the <font color=#0000ff>query</font> molecule in <font color=#0000ff>blue</font> and the <font color=#ff0000>target</font> in <font color=#ff0000>red</font>.<br><br><hr><br>$EOL"; 

    print htmlfile "<p><b>LOCK 2 search results for query: <a href=$querypdblink>$queryoutname</a></b><br><br>$EOL";

    $errorString = `/bin/grep -i sorry $outputdir/logfile`;

    if ($errorString ne "") {
	open(errorfile, ">$outputdir/errorfile");
	print errorfile $errorString;
	close(errorfile);
	print htmlfile "<b>!!! Some <a href=$wwwoutput/errorfile>ERROR(s)</a> occurred !!!</b><br><br>$EOL";
    }
	

    print htmlfile "<a href=$wwwoutput/logfile>Execution Log</a><br>$EOL";
    print htmlfile "<a href=$wwwoutput/search\.sort>Search results (non-html)</a><br>$EOL";

    print htmlfile "<br>$EOL";
	
    open(INFILE, "<$outputdir/search\.sort");
    print htmlfile "<table width=100% cellpadding=0 cellspacing=4 border=0>$EOL";
    print htmlfile "<tr>$EOL";
    print htmlfile "<td align=left> <b>Target</b> </td>$EOL";
    print htmlfile "<td align=center> <b><a href=\"javascript:launchHelpWindow('resultsformats')\" onMouseOver=\"status='Click for help';return true\" onMouseOut=\"status='';return true\" class=\"helplink\">Results</a></b> </td>$EOL";
    print htmlfile "<td align = center> <b><a href=\"javascript:launchHelpWindow('alignmentscore')\" onMouseOver=\"status='Click for help';return true\" onMouseOut=\"status='';return true\" class=\"helplink\">Score</a></b> </td>$EOL"; #added
    print htmlfile "<td align = center> <b><a href=\"javascript:launchHelpWindow('significance')\" onMouseOver=\"status='Click for help';return true\" onMouseOut=\"status='';return true\" class=\"helplink\">P value</a></b> </td>$EOL"; #added
    print htmlfile "<td align = center> <b>SSEs Aligned</b></td>$EOL"; #added
    print htmlfile "<td align=center> <b>CA Atoms Aligned</b> </td>$EOL";
    print htmlfile "<td align=center> <b>RMSD</b> </td>$EOL";
    print htmlfile "<td align=left> <b>PDB HEADER</b> </td>$EOL";
    print htmlfile "</tr>$EOL";
    while ($line = <INFILE>) {
	printlongtableline($line);
    }
    print htmlfile "</table>$EOL";
    close(INFILE);
    system("$mv","$outputdir/tmp","$outputdir/results\.html");
    printMultSearchHeader();
}


sub printlongtableline() {

    chop($_[0]);
    @fields = split /\|/, $_[0];

    if ($fields[1] >= $minret) {
	$target = $fields[0];
	if(length($target) == 7) { #is SCOP id
	  $targetpdblink = "http://scop.berkeley.edu/search.cgi?sid=$target";
	}else {
	  $targetpdblink =  "/cgi-bin/FoldMiner2/browse2.cgi?pdbname=$target&resultsdir=$outputdir&wwwdir=$wwwoutput";
	}	
	print htmlfile "<tr>$EOL";
	print htmlfile "<td align=left>$EOL";
	print htmlfile "<a href=$targetpdblink>$target</a></td>$EOL";
	print htmlfile "<td align=center>$EOL";
	print htmlfile "<a href=$wwwoutput/$queryoutname\_$target\.out>Text</a>";
	print htmlfile " , ";
	print htmlfile "<a href=/cgi-bin/FoldMiner2/newpdb.cgi?yourfile=$queryoutname\_$target\.out\&outfile=$queryoutname\_$target\.pdb\&outdir=$outputdir\&wwwdir=$wwwoutput\&chime=0>PDB</a>";

	print htmlfile " , ";
	print htmlfile "<a href=/cgi-bin/FoldMiner2/newpdb.cgi?yourfile=$queryoutname\_$target\.out\&outfile=$queryoutname\_$target\.pdb\&outdir=$outputdir\&wwwdir=$wwwoutput\&chime=1>Chime</a>";


	print htmlfile "</td>$EOL";
	print htmlfile "<td align=center>$EOL";
	print htmlfile "$fields[1]</td>$EOL";
	$pval = &expectation_function($lambda{$queryfold},$mu{$queryfold},$fields[1]);	
	printf(htmlfile "<td align=center>%8.3g</td>$EOL",$pval);
	print htmlfile "<td align=center>$EOL";
	print htmlfile "$fields[2]</td>$EOL";
	print htmlfile "<td align=center>$EOL";
	print htmlfile "$fields[3]</td>$EOL";
	print htmlfile "<td align=center>$EOL";
	print htmlfile "$fields[4]</td>$EOL";
	print htmlfile "<td align=left>$EOL";
	print htmlfile "$fields[5]</td>$EOL";

	print htmlfile "</td>$EOL";
	print htmlfile "</tr>$EOL";
    }
}


sub runlock_search() {
  my($expectation,$error);

  if(not defined $address) {
    $address = "NONE";
  }

  $expectation = $base * (10**$exponent); #compute expectation
  open(tempout, ">$outputdir/options.list");

  print tempout "$outputdir\n";
  print tempout "$wwwoutput\n";
  print tempout "$query\n";
  print tempout "$querychainopt $querychain\n";
  print tempout "$target\n";
  print tempout "$seqpairsoption\n";
  print tempout "$distthreshold\n";
  print tempout "$queryoutname\n";
  print tempout "$querypdblink\n"; #make this scop link if scopid is used?
  print tempout "$address\n";
  print tempout "$expectation\n";
  print tempout "$xval\n";
  print tempout "$base\n";
  print tempout "$exponent\n";
  print tempout "$chime\n";
  close(tempout);

  if($pdbname1 ne "") {
    
    system("./findfile.pl",$pdbname1,"$outputdir/$query\.pdb");
    if(! -e("$outputdir/$query\.pdb")) {
      redirectToErrorHandler($pdbname1,"The query PDB file could not be located in our local database. If there are no errors in your input, please repeat your request and upload the PDB file.","d1f5wa1,1f5w,1f5w-A","query",$ID);
	  exit(-1);
	}

    }else { #must have uploaded the file: need to deal with this option
 	#MAKE SURE FILE IS IN CORRECT DIRECTORY AND IS NAMED APPROPRIATELY
    }	
    
    printSearchHeader();
  $error = system("echo '1' > $outputdir/filenum.txt");
#  close(STDOUT); #makes sure response is loaded

    $error =system("/var/www/cgi-bin/FoldMiner2/runlock.pl", "$outputdir/options.list", "\>", "/dev/null", "\&");
}
sub printMultSearchHeader() {
print <<END;
HTTP/1.0 302 Found
Server:  Apache/1.3.27 (Unix)  (Red-Hat/Linux)
MIME-version: 1.0
Location: $wwwoutput/results.html
Cache-control: no-cache
Content-type: text/html

<html>
<head>
<meta http-equiv=description name=description 
content="LOCK, output">
<meta http-equiv=keywords name=keywords 
content="hierarchical protein structure superposition, bioinformatics, LOCK, biochemistry">

<title>LOCK 2 Results: Processing...</title>
</head>

END

}

sub printSearchHeader() {

print <<END;
HTTP/1.0 302 Found
Server:  Apache/1.3.27 (Unix)  (Red-Hat/Linux)
MIME-version: 1.0
Location: /cgi-bin/FoldMiner2/findmotif.cgi?id=$ID&expectation=$expectation&xval=$xval&chime=$chime&db=$target
Cache-control: no-cache
Content-type: text/html

<html>
<head>
<meta http-equiv=description name=description 
content="LOCK, output, FoldMiner">
<meta http-equiv=keywords name=keywords 
content="hierarchical protein structure superposition, bioinformatics, LOCK, biochemistry, FoldMiner">

<title>FoldMiner Results: Processing...</title>
</head>
END
}


sub printHeader() {
print <<END;
HTTP/1.0 302 Found
Server:  Apache/1.3.27 (Unix)  (Red-Hat/Linux)
MIME-version: 1.0
Location: $wwwoutput/results.html
Cache-control: no-cache
Content-type: text/html

<html>
<head>
<meta http-equiv=description name=description 
content="LOCK 2, output">
<meta http-equiv=keywords name=keywords 
content="hierarchical protein structure superposition, bioinformatics, LOCK, biochemistry">

<title>LOCK 2 Results: Done</title>
</head>

<body bgcolor=#FFFFFF link=#26246b alink=#8777bab vlink=#8777bab>

<p>

<table width=100% cellpadding=0 cellspacing=0 border=0>

<tr><td align=left><a href=/FoldMiner2/?lock target=_top><img align=left src=/FoldMiner2/images/lock2.gif border=0></a></td>

<td align=right>
<br><a href="mailto:jessicas\@stanford.edu" >Jessica Shapiro</a>
<br><a href=http://gene.stanford.edu/~apsingh target=_top>Amit P. Singh</a>
<br><a href=//cmgm.stanford.edu/~brutlag target=_top>Douglas L. Brutlag</a>
<br><a href=http://motif.stanford.edu target=_top>Bioinformatics Group</a>
<br><a href=//www.stanford.edu/ target=_top>Stanford University</a>
</td>
</tr>
</table></p>

<br>
END
}


sub printHtmlHeader() {

print htmlfile <<END;

<html>
<head>
<meta http-equiv=description name=description 
content="LOCK, output">
<meta http-equiv=keywords name=keywords 
content="hierarchical protein structure superposition, bioinformatics, LOCK, biochemistry">

<title>LOCK 2 Results: Done</title>
<link rel=stylesheet href="/FoldMiner2/lock.css" type="text/css">
<script language="Javascript" type="text/javascript">
  function launchHelpWindow(anchor) {
   var URL = "/FoldMiner2/help.html#"+anchor;
   var mywin=window.open('','help',"directories=no,location=no,menubar=no,scrollbars=yes,toolbar=no,resizable=yes,status=yes,width=350,height=300");
   mywin.location.replace(URL);
   mywin.focus();
  }
</script>
</head>

<body bgcolor=#FFFFFF link=#26246b alink=#8777bab vlink=#8777bab>

<p>
<table width=100% cellpadding=0 cellspacing=0 border=0>

<tr><td align=left><a href=/FoldMiner2/?lock target=_top><img align=left src=/FoldMiner2/images/lock2.gif border=0></a></td>

<td align=right>
<br><a href="mailto:jessicas\@stanford.edu" >Jessica Shapiro</a>
<br><a href=http://gene.stanford.edu/~apsingh target=_top>Amit P. Singh</a>
<br><a href=//cmgm.stanford.edu/~brutlag target=_top>Douglas L. Brutlag</a>
<br><a href=http://motif.stanford.edu target=_top>Bioinformatics Group</a>
<br><a href=//www.stanford.edu/ target=_top>Stanford University</a>
</td>
</tr>
</table></p>


<br>
END
}


sub securityCheckProteinName {
  my($inputname, $isScop) = @_;
  my($cleanname,$pdbid);
  
  $cleanname = "";
  if($isScop) {
    if($inputname =~ /^([dg][0-9][A-Za-z0-9]{3}[\._A-Za-z0-9][_0-9])/) {
      $cleanname = $1;
    }else {
      $cleanname = "";
    }
  }else {
    if($inputname =~ /^([0-9][A-Za-z0-9]{3})$/) {
      $cleanname = $1;
    }elsif($inputname =~/^([0-9][A-Za-z0-9]{3})(-[A-Za-z0-9])/) {
      $cleanname = $1.$2; #$1 is the pdb id, and $2 is the chain, prefixed by '-'
    }elsif($inputname =~/^([0-9][A-Za-z0-9]{3})([A-Za-z0-9])/) { #user attempted to specify chain without '-'
      $cleanname = $1."-".$2;
    }else {
      $cleanname == "";
    }
  }

  return $cleanname; 
}

sub redirectToErrorHandler {
  my($input,$error,$correctinput,$helptopic,$pid) = @_;
  my($email) = "jessicas\@stanford.edu";
  my($name) = "Jessica Ebert";
  my($runtype);

  $input=CGI::escape($input);
  $error = CGI::escape($error);
  $correctinput=CGI::escape($correctinput);
  $helptopic=CGI::escape($helptopic);
  $email=CGI::escape($email);
  $name=CGI::escape($name);

  if($multsearch) {
    $runtype = "multiple";
  }elsif($searchpdb) {
    $runtype = "search";
  }else {
    $runtype = "single";
  }

  print <<END;
HTTP/1.0 302 Found
Server:  Apache/1.3.27 (Unix)  (Red-Hat/Linux)
MIME-version: 1.0
Cache-control: no-cache
Location: /cgi-bin/FoldMiner2/reporterror.cgi?error=$error&exampleinput=$correctinput&input=$input&email=$email&name=$name&helptopic=$helptopic&resultsdir=$pid&runtype=$runtype
END

}


#Returns the value of the CDF of extreme value distribution for 
#parameters lambda and mu at x
sub expectation_function {
  my($lambda,$mu,$x) = @_;

  return(1-exp(-exp(-$lambda*($x-$mu))));
}
