#!/usr/bin/perl -T

# Copyright 2003 The Board of Trustees of 
# The Leland Stanford Junior University. All Rights Reserved.

# Communication to Jessica Shapiro, jessicas@stanford.edu

####################################################
#reporterror.cgi: A CGI script used in the FoldMiner web pages to report errors to the
#user. It will report that an error occurred, and if further information is supplied
#via the parameters (see below), additional information information can be presented
#to the user. Such information could include the user's incorrect input (most FoldMiner
#errors come from filling out forms incorrectly, examples of acceptable input,
#and links to help pages. I have this set up right now to interface with the FoldMiner
#help pages, so others may need to change the code that deals with the $helpanchor
#variable.
#
#Note on parameters: It is convenient in a programming context to pass all parameters
#even when some are not used. If you don't want to use a certain parameter but want
#to pass it to the script for the sake of clarity, pass empty strings. These will be
#ignored.
#
#Some parts of this script are specific to FoldMiner (headers that are printed
#on every error report, style sheet link, etc). 
#######################################################

use strict vars; #requires variables to be declared
use CGI;

print "Content-type: text/html\n\n";

$ENV{'PATH'} = "/bin:/usr/bin:/usr/local/bin"; #path must be set in taint mode

#some variable declarations
my($touch);
my($errorstring,$exampleinput,$runtype,$helpanchor,$exampleurl,$email,$resultsdir,$yourinput,$name,$image);
my($outputdir);
my($q) = new CGI;

$touch = "/bin/touch";
$errorstring = $q->param('error'); #error to report to user
$yourinput = $q->param('input'); #the user's input
$runtype = $q->param('runtype'); #type of run: query vs. target, similarity search, etc. (not necessary)
$exampleinput = $q->param('exampleinput'); #example of correct syntax, if applicable
$helpanchor = $q->param('helptopic'); #anchor to help topic, if applicable
$exampleurl = $q->param('exampleurl'); #url with example, if applicable
$resultsdir = $q->param('resultsdir'); #user's results directory, if it exists yet
$email = $q->param('email'); #contact email address for contact person
$name = $q->param('name'); #name of contact person

#need to check resultsdir since we'll use it to open a file
if($resultsdir =~ /^([0-9]+)$/) {
  $resultsdir = $1;
  $outputdir = "/var/www/html/FoldMiner2/$resultsdir";
}else {
  $resultsdir = "";
}

if(!defined($runtype)) {
  $runtype = 0;
}

#multiple example inputs may be sent as a comma separated list
my(@exampleinputs,$input);
if(defined($exampleinput)) {
  @exampleinputs = split(",",$exampleinput); #could have more than 1
}


my($program);
if(defined($runtype) && $runtype ne "") {
  if($runtype eq "search") {
    $image = "foldminer.gif";
    $program = "FoldMiner";
  }else {
    $image = "lock2.gif";
    $program = "lock 2";
    $program =~ tr/A-Z/a-z/;
  }
}else {
  $image = "lock2.gif";
  $program = "lock 2";
  $program =~ tr/A-Z/a-z/;
}


#print header stuff (LOCK logo, etc)
print <<END;
<html>
<head>
<link rel=stylesheet href="/FoldMiner2/lock.css" type="text/css"> 
<title>$program: Error</title>
<script language="Javascript">

function launchHelpWindow(anchor) {
   var URL = "/FoldMiner2/help.html#"+anchor;
   var mywin=window.open(URL,'help',"directories=no,location=no,menubar=no,scrollbars=yes,toolbar=no,resizable=yes,status=yes,width=350,height=300");
    mywin.focus();
}
</script>
</head>
<body bgcolor=#ffffff link=#000066 alink=#44488 vlink=#44488>
<p>
<table width=100% cellpadding=0 cellspacing=0 border=0>
END

if(defined($runtype) && $runtype ne "") {
    if($runtype eq "multiple" || $runtype eq "single") {
          print "<tr><td align=left><a href=/FoldMiner2/?lock target=_top><img align=left src=/FoldMiner2/images/$image border=0></a></td>\n";
     }else {
          print "<tr><td align=left><a href=/FoldMiner2/ target=_top><img align=left src=/FoldMiner2/images/$image border=0></a></td>\n";
     }
}

print <<END;
<td align=right>
<br><a href="mailto:jessicas\@stanford.edu" >Jessica Shapiro</a>
<br><a href=http://gene.stanford.edu/~apsingh target=_top>Amit P. Singh</a>
<br><a href=//cmgm.stanford.edu/~brutlag target=_top>Douglas L. Brutlag</a>
<br><a href=http://motif.stanford.edu target=_top>Bioinformatics Group</a>
<br><a href=//www.stanford.edu/ target=_top>Stanford University</a>
</td>
</tr>
</table></p>
END

#print the error string and the run type information (if it exists)
if(defined($runtype) && $runtype ne "") {
  if($runtype eq "multiple") {
    print "An error occurred while you were aligning a query structure to multiple targets: ";
  }elsif($runtype eq "single") {
    print "An error occurred while you were aligning a query structure to a target: ";
  }elsif($runtype eq "search") {
    print "An error occurred while you were aligning a query structure to a database of targets: "
  }else {
    print "An error occurred: ";
  }
}else {
  print "An error occurred: ";
}

print "$errorstring<br>\n";


#if user's input was passed to this script, print it out
if(defined($yourinput) && $yourinput ne "") {
  print "<br>Your input was: <code>$yourinput</code><br>\n";
}

#if example input was passed to the script, print it out
if(defined($exampleinput) && $exampleinput ne "") {
  print "<br>\n";
  if($#exampleinputs == 0) { #only one example input was provided
    $exampleinputs[0] =~ s/\n/<br>/g; #input could have newline characters in it
    $exampleinputs[0] = "<br>".$exampleinputs[0];
    $exampleinputs[0].="<br>";
    print "Correct input looks like this: $exampleinputs[0]\n";
  }else {
    #Multiple sample inputs were provided. Display as a list
    print "Correct input follows one of these patterns:<ul>\n";
    foreach $input (@exampleinputs) {
      $input =~ s/\\n/<br>/g; #input could have newline characters in it
      print "<li><code>$input</code>\n";
    }
    print "</ul>\n";
  }
  print "<br>\n";
}

#All of the LOCK 2.0 help information is in one large file, and I use a javascript function to launch the help window. There's no need to use the javascript function here; it's just consistent with the rest of the LOCK 2.0 web pages. Substitute your own URL and remove/adjust the javascript
if(defined($helpanchor) && $helpanchor ne "") {

  print "<a href=\"javascript:launchHelpWindow(\'$helpanchor\')\" onMouseOver=\"status='Click for help';return true\" onMouseOut=\"status='';return true\" class=\"helplink\">Click here to review a help topic on this subject.</a><br>\n"
}

#If you would like to provide the user with a link to a form that is correctly filled in, or a link to an example run, it can be passed in through the exampleurl parameter
if(defined($exampleurl) && $exampleurl ne "") {
  print "<a href=\"$exampleurl\">Run an example</a><br>\n";
}

#indicate that we will fix bugs
print "<br>The FoldMiner server is consistently monitored for errors such as this. If you have encountered a bug, it will most likely be fixed within the next day or two. Please help speed up this process by submitting a bug report.\n<br><br>";

#Use the email and name parameters to ask the user to send email (for bug reports, etc). In the case of LOCK 2.0, the user is assigned a specific results directory, so I ask users to report the ID number (resultsdir) that identifies that directory. If this parameter isn't supplied, this block of code will still function correctly.
if(defined($email) && defined($name)) {
#  $email =~ s/\@/\\\@/; #replace the @ character
  print <<END;
<p>If you believe you reached this page in error, please send email to <a href='mailto:$email'>$name</a>. Include the error message
END
  
  if(defined($resultsdir) && $resultsdir ne "") {
    print ", as much information as possible about what you were doing when the error occurred, and this number: $resultsdir.<br>\n";
    system($touch,"$outputdir/done");
  }else {
    print " and as much information as possible about what you were doing when the error occurred.<br>\n";
  }
}
print "</body></html>\n";

if(defined($resultsdir) && $resultsdir ne "") {
  open(ERROR, ">$resultsdir/error.txt");
  print ERROR "$runtype\n$yourinput\n$exampleinput\n$helpanchor\n";
  if(defined($email) && $email ne "") {
    print ERROR "email: $email\n";
  }
  close(ERROR);
}
